#!/bin/env python3

import sys, mysql.connector, json
from io import StringIO
from operator import itemgetter

db = sys.argv[1]

from configparser import RawConfigParser as ConfigParser
parser = ConfigParser()
parser.read([sys.argv[2]])
if not parser.has_section('main'):
    raise KeyError("[main] section missing")

spec = {}
for sec in parser.sections():
    if sec == "main":
        if not parser.has_option('main','query'):
            raise KeyError("[main] section missing query parameter")
        spec['query'] = parser.get('main','query')
    else:
        if not parser.has_option(sec,'query'):
            raise KeyError("[%s] section missing query parameter"%(sec,))
        if not parser.has_option(sec,'params'):
            raise KeyError("[%s] section missing params parameter"%(sec,))
        spec[sec] = dict(query=parser.get(sec,'query'),
                         params=parser.get(sec,'params').split())

db = mysql.connector.connect(host="127.0.0.1",user="root",db=db,port=3306)

def execute(db,spec,query=None,params=()):
  root = False
  if not query:
    query = spec['query']
    root = True
  cursor = db.cursor()
  cursor.execute(query,params)
  rows = []; headers = None
  for r in cursor.fetchall():
    if headers == None:
      headers = list(map(str,map(itemgetter(0),cursor.description)))
    row = dict(zip(headers,r))
    if root and '_id' not in row:
      raise KeyError("Main query must provide an _id field.")
    for k,v in list(row.items()):
      if isinstance(v,str) and v.startswith('jsonlist:'):
        key = v.split(':',1)[1]
        params = tuple(map(row.get,spec[key]['params']))
        subquery = spec[key]['query']
        v = execute(db,spec,subquery,params)
        row[k] = list(map(lambda d: list(d.values())[0],v))
      if isinstance(v,str) and v.startswith('jsonselect:'):
        key = v.split(':',1)[1]
        params = tuple(map(row.get,spec[key]['params']))
        subquery = spec[key]['query']
        v = execute(db,spec,subquery,params)
        row[k] = v
    if root:
      print(row["_id"],file=sys.stderr)
    rows.append(row)
  return rows

rows = execute(db,spec)
print(json.dumps(dict(docs=rows),indent=2,sort_keys=True,skipkeys=True))

